/***************************************************************************
 *   Copyright (C) 2006 by Julien Lemoine, Simon Viennot                   *
 *   lapinot@tuxfamily.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <fstream>
#include <sstream>
#include <QApplication>
#include "widgetprincipal.h"
#include "soustirage.h"
#include "jeu.h"
#include "kitsune.h"

void main_aide();
void main_compte();
void main_erreur();
void main_renard();
void main_stats();
void main_stats2();	//fonction de test pour statistiques limites
void main_test();	//fonction temporaire servant  raliser des tests lors de la programmation

plaque demande_plaque();  //fonction pour l'entre d'une plaque en ligne de commande
compte demande_compte();  //fonction pour l'entre du compte en ligne de commande

int main( int argc, char **argv ) {
	string s;
	switch(argc) {
	  case 2 : s=(string) argv[1];
		if     (s=="-h" || s=="--help")		{main_aide();}
		else if(s=="-c" || s=="--compte")	{main_compte();}
		else if(s=="-s" || s=="--stats")	{main_stats();}
		//else if(s=="-s2" || s=="--stats2")	{main_stats2();}
		else if(s=="-r" || s=="--renard")	{main_renard();}
		//else if(s=="-t" || s=="--test")		{main_test();}	//fct temporaire pour des tests lors de la programmation
		else					{main_erreur();}
		break;
	  case 1 : {QApplication a( argc, argv );
		Principal *hm = new Principal();
		hm->show();
		return a.exec();} break;
	  default : main_erreur();
	}

	return EXIT_SUCCESS;
}

void main_aide() {
	cout << "Liste des options disponibles pour le programme Kitsune :" << endl << endl
	     << "rien           :" << "  " << "lance l'interface graphique du programme." << endl
	     << "-c ou --compte :" << "  " << "rsout un problme particulier en ligne de commande." << endl
	     << "-h ou --help   :" << "  " << "affiche une liste des paramtres disponibles." << endl
	     << "-r ou --renard :" << "  " << "?" << endl
	     << "-s ou --stats  :" << "  " << "lance une srie de calculs statistiques." << endl;
}

void main_erreur(){
	cout << "Paramtre inexistant." << endl
	     << "Utiliser l'option \"-h\" ou \"--help\" pour avoir une liste des paramtres disponibles." << endl;
}

void main_compte(){
	plaque pl;
	compte compte_init;

	string erreur_tirage="Erreur : tirage non valide, il ne faut pas plus de deux plaques de chaque type entre 1 et 10, et pas plus d'une plaque de chaque type pour 25, 50, 75, 100.";
	
	string noms[6]={"Premire plaque  : ", "Deuxime plaque  : ", "Troisime plaque : ",
			"Quatrime plaque : ", "Cinquime plaque : ", "Sixime plaque   : "};
	tirage t;
	
	for(int i=0; i<6; i++) {
		cout<< noms[i];
		pl=demande_plaque();
		t[pl]++;
	}

	if(!est_valable(t)) { 
		cout << erreur_tirage << endl; 
		return;
	}
	
	cout<<"         compte  : ";
	compte_init=demande_compte();
	
	jeu J;
	J.etude_tirage(t,compte_init,compte_init);
	cout << endl;
	cout << J.afficheBase();
}

plaque demande_plaque(){
	string erreur_plaque="         Erreur  : plaques possibles : 1 - 10, 25, 50, 75, 100.";
	
	int i; plaque pl;
	string s;

	while(true) {
		getline(cin,s);
		istringstream istr(s);
		if(istr>>i) { 
			if ( (i>=1 && i<=10) || i==25 || i==50 || i==75|| i==100) {
				pl=i;
				return pl;
			}
		}
		
		//message erreur et nouvelle demande de plaque
		cout<< erreur_plaque << endl << "         plaque  : ";
	}
}

compte demande_compte(){
	string erreur_compte="         Erreur  : rentrer un nombre entier entre 100 et 999.";
	
	int i; compte resultat;
	string s;
	
	while(true) {
		getline(cin,s);
		istringstream istr(s);
		if(istr>>i) { 
			if (i<=999 && i>=100) {
				resultat=i;
				return resultat;
			}
		}
		
		//message erreur et nouvelle demande du compte
		cout<< erreur_compte << endl << "         compte  : ";
	}
}

void main_renard(){
	cout << "    _   _" << endl
	     << "   | \\_/ |" << endl
	     << "   /     \\     /^^^/" << endl
	     << "  /_.^ ^._\\   /^^^|" << endl
	     << "     \\o/      |   |" << endl
	     << "    /   \\_    |   /" << endl
	     << "    \\     \\_  /  /" << endl
	     << "     | | /  \\/ _/" << endl
	     << "    _|_|_\\ _/_/" << endl;
}

//tude limite  un petit nombre de tirages (pour les tests)
void main_stats2(){
	tirage t1;
	t1[1]=1; t1[2]=1; t1[3]=2; t1[100]=1; t1[75]=1;
	
	tirage t2;
	t2[1]=1; t2[2]=1; t2[3]=1; t2[4]=1; t2[5]=1; t2[100]=1;
	
	tirage t3;
	t3[1]=2; t3[2]=2; t3[3]=2;
	
	set<tirage> st;
	st.insert(t1);
	st.insert(t2);
	st.insert(t3);
	
	etude_globale E;
	E.grosse_etude(st);
}

void main_stats(){
//test pour dterminer si on a le droit de crer un fichier
	ofstream f_premier( "premier.txt" );
	if ( !f_premier )
	{
	cerr << "Erreur : impossible de crer un fichier dans ce rpertoire.\n";
	return;
	}
	f_premier.close();
//pour lancer l'tude statistique globale sur les 13243 tirages
	set<tirage> st=tous_les_tirages();
	etude_globale E;
	E.grosse_etude(st);
}

//fonction temporaire servant  raliser des tests lors de la programmation
void main_test(){
	calcul Calc=calcul(6)*calcul(5)*calcul(2)/calcul(4);
	//calcul Calc=(calcul(5)+calcul(1))*calcul(6)*calcul(4)*calcul(3)*calcul(2);
	list<compte> fugo=Calc.liste_sous_comptes();
	list<compte>::iterator gougou;
	for(gougou=fugo.begin();gougou!=fugo.end();gougou++){
		cout << *gougou << "\t" ;
	}
	cout << endl;
}

//Vieux truc qui pourrait redevenir utile :
//Dcommenter ce qui suit pour tudier tous les tirages et afficher le nombre de comptes raliss par chacun
//Ceci peut tre utile notamment pour tester une amlioration dans les solutions quivalentes

/*
	set<tirage> plop=tous_les_tirages();
	set<tirage>::iterator TIR;
	
	for(TIR=plop.begin();TIR!=plop.end();TIR++){
	
	jeu J;
	J.etude_tirage(*TIR,100,999);
	cout << *TIR << "\t" << J.nb_comptes_realises() << "\t" << J.nb_calculs_stockes() << endl;
	
	}
*/
